/*=============================================================================
    Copyright (c) 2002-2003 Hartmut Kaiser
    http://spirit.sourceforge.net/

    Use, modification and distribution is subject to the Boost Software
    License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
    http://www.boost.org/LICENSE_1_0.txt)
=============================================================================*/
#ifndef BOOST_SPIRIT_XPRESSIVE_IPP
#define BOOST_SPIRIT_XPRESSIVE_IPP

///////////////////////////////////////////////////////////////////////////////
#include <boost/spirit/core/primitives/impl/primitives.ipp>

///////////////////////////////////////////////////////////////////////////////
namespace boost { namespace spirit {

namespace impl {

///////////////////////////////////////////////////////////////////////////////
//
//  xpressive_parser class
//
///////////////////////////////////////////////////////////////////////////////
template <typename IteratorT>
class xpressive_parser 
:   public parser<xpressive_parser<IteratorT> > 
{
private:
    typedef typename std::iterator_traits<IteratorT>::value_type char_t;
    typedef std::basic_string<char_t> string_t;

public:
    typedef xpressive_parser<IteratorT> self_t;

    template <typename ScannerT>
    struct result
    {
        typedef xpressive::basic_match_results<IteratorT> result_t;
        typedef typename match_result<ScannerT, result_t>::type type;
    };

    xpressive_parser(IteratorT first, IteratorT last)
    { pattern = xpressive::begin >> string_t(first, last); }

    xpressive_parser(IteratorT first)
    { pattern = xpressive::begin >> string_t(first, impl::get_last(first)); }

    xpressive_parser(xpressive::basic_regex<IteratorT> const &pattern_)
    :   pattern(pattern_) {}

    template <typename ScannerT>
    typename parser_result<self_t, ScannerT>::type
    parse(ScannerT const& scan) const
    {
        xpressive::basic_match_results<IteratorT> result;
        bool matched = xpressive::match(pattern, scan.first, scan.last, result);

        if (!matched)
            return scan.no_match();

        scan.first = result[0].second;
        return scan.create_match(result[0].length(), result, result[0].first, 
          scan.first);
    }

private:
    xpressive::basic_regex<IteratorT> pattern;
};

}   // namespace impl

///////////////////////////////////////////////////////////////////////////////
}} // namespace boost::spirit

#endif // BOOST_SPIRIT_XPRESSIVE_IPP
